# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/requests/transaction/create_transaction_response.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.transaction import transaction_pb2 as fintekkers_dot_models_dot_transaction_dot_transaction__pb2
from fintekkers.requests.transaction import create_transaction_request_pb2 as fintekkers_dot_requests_dot_transaction_dot_create__transaction__request__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAfintekkers/requests/transaction/create_transaction_response.proto\x12\x1f\x66intekkers.requests.transaction\x1a/fintekkers/models/transaction/transaction.proto\x1a@fintekkers/requests/transaction/create_transaction_request.proto\"\xfa\x01\n\x1e\x43reateTransactionResponseProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x62\n\x1a\x63reate_transaction_request\x18\x14 \x01(\x0b\x32>.fintekkers.requests.transaction.CreateTransactionRequestProto\x12M\n\x14transaction_response\x18\x1e \x01(\x0b\x32/.fintekkers.models.transaction.TransactionProtoB#B\x1f\x43reateTransactionResponseProtosP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.transaction.create_transaction_response_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\037CreateTransactionResponseProtosP\001'
  _CREATETRANSACTIONRESPONSEPROTO._serialized_start=218
  _CREATETRANSACTIONRESPONSEPROTO._serialized_end=468
# @@protoc_insertion_point(module_scope)




from fintekkers.models.util.local_date_pb2 import LocalDateProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.models.transaction.transaction_type_pb2 import TransactionTypeProto
from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto

from fintekkers.wrappers.models.util.fintekkers_uuid import FintekkersUuid
from fintekkers.wrappers.models.security import Identifier
from fintekkers.wrappers.models.transaction import TransactionType

from google.protobuf.timestamp_pb2 import Timestamp
from datetime import date, datetime
import time
from pytz import timezone

from uuid import UUID, uuid4

class ProtoEnum:
    def __init__(self, enum_name:str, enum_value:int):
        self.enum_name:str = enum_name
        self.enum_value:int = enum_value

class ProtoSerializationUtil:
    @staticmethod
    def serialize(obj):
        if isinstance(obj, UUID):
            return UUIDProto(raw_uuid=obj.bytes)
        if type(obj) is date:
            return LocalDateProto(year=obj.year, month=obj.month, day=obj.day)
        if isinstance(obj, datetime):
            seconds_in_float:float = time.mktime(obj.timetuple())
            timestamp = Timestamp(seconds=int(seconds_in_float), 
                nanos=int(obj.microsecond/1000))

            time_zone = obj.timetz().tzinfo.zone if obj.timetz().tzinfo is not None else "America/New_York"
            return LocalTimestampProto(timestamp=timestamp, time_zone=time_zone)
        if isinstance(obj, Identifier) or isinstance(obj, TransactionType):
            return obj.proto

        return obj
        

    @staticmethod
    def deserialize(obj):
        if isinstance(obj, UUIDProto):
            return FintekkersUuid.from_bytes(bytes=obj.raw_uuid)
        if isinstance(obj, LocalDateProto):
            return date(year=obj.year, month=obj.month, day=obj.day)
        if isinstance(obj, LocalTimestampProto):
            return datetime.fromtimestamp(obj.timestamp.seconds, timezone(obj.time_zone))
        if isinstance(obj, IdentifierProto):
            return Identifier(obj)
        if hasattr(obj, 'enum_name') and getattr(obj, 'enum_name') == "TRANSACTION_TYPE":
            return TransactionType(obj.enum_value)
        
        return obj

if __name__ == "__main__":
    serialized:UUIDProto = ProtoSerializationUtil.serialize(uuid4())
    assert isinstance(serialized, UUIDProto)
    deserialized:UUID = ProtoSerializationUtil.deserialize(serialized)
    assert isinstance(deserialized, UUID)

    serialized:LocalDateProto = ProtoSerializationUtil.serialize(date.today())
    assert isinstance(serialized, LocalDateProto)
    deserialized:date = ProtoSerializationUtil.deserialize(serialized)
    assert isinstance(deserialized, date)

    obj = datetime.today().replace(tzinfo=timezone("America/New_York"))
    serialized:LocalTimestampProto = ProtoSerializationUtil.serialize(obj)
    assert isinstance(serialized, LocalTimestampProto)
    deserialized:datetime = ProtoSerializationUtil.deserialize(serialized)
    assert isinstance(deserialized, datetime)



from setuptools import setup, find_packages

import os

version = '0.0.0'
if 'BUILD_VERSION' in os.environ:
    version = os.environ.get('BUILD_VERSION')

setup(
    name = "fintekkers-ledger-models",
    version=version,
    license='MIT',
    author="David Doherty",
    author_email='davidjdoherty@gmail.com.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/fintekkers/ledger-models',
    keywords='example project',
    install_requires=[
      ],

)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/models/util/lock/node_partition.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0fintekkers/models/util/lock/node_partition.proto\x12\x1b\x66intekkers.models.util.lock\"\\\n\rNodePartition\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x11\n\tpartition\x18\x03 \x01(\x05\x12\x11\n\tnamespace\x18\x04 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.util.lock.node_partition_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _NODEPARTITION._serialized_start=81
  _NODEPARTITION._serialized_end=173
# @@protoc_insertion_point(module_scope)



from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.measure_pb2 import MeasureProto
from fintekkers.models.position.position_pb2 import PositionProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry, MeasureMapEntry

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.security.security_pb2 import SecurityProto
from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto

from fintekkers.models.util.local_date_pb2 import LocalDateProto
from fintekkers.models.util.uuid_pb2 import UUIDProto

from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil, ProtoEnum

from google.protobuf import wrappers_pb2 as wrappers
from google.protobuf.any_pb2 import Any

from decimal import Decimal

class Position():
    positionProto:PositionProto

    def __init__(self, positionProto:PositionProto) -> None:
        self.positionProto = positionProto

    def get_field(self, field_to_get:FieldProto):
        tmp_field:FieldMapEntry
        for tmp_field in self.positionProto.fields:
            if tmp_field.field == field_to_get:
                return ProtoSerializationUtil.deserialize(Position.unpack_field(tmp_field))

        raise ValueError("Could not find field in position")
        

    def get_measure(self, measure_to_get:MeasureProto):
        tmp_measure:MeasureMapEntry
        for tmp_measure in self.positionProto.measures:
            if tmp_measure.measure == measure_to_get:
                return ProtoSerializationUtil.deserialize(Position.unpack_measure(tmp_measure))


        raise ValueError("Could not find measure in position")

    def get_field_display(self, field_to_get:FieldProto):
        return self.get_field(field_to_get=field_to_get).__str__()

    def get_measures(self):
        return self.positionProto.measures

    def get_fields(self):
        return self.positionProto.fields

    @staticmethod
    def wrap_string_to_any(my_string:str):
        my_any = Any()
        my_any.Pack(wrappers.StringValue(value=my_string))
        return my_any

    # @staticmethod
    # def unwrap_string_from_any(msg):
    #     any:Any = Any()
    #     any.Unpack(msg)
    #     return any.value


    @staticmethod
    def pack_field(field_to_pack):
        if field_to_pack.__class__ == LocalDateProto:
            my_any = Any()
            my_any.Pack(field_to_pack)
            return my_any

        # if field_to_unpack.field == FieldProto.PORTFOLIO_ID:
        #     return UUIDProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.TRADE_DATE or field_to_unpack.field == FieldProto.SETTLEMENT_DATE \
        #     or field_to_unpack.field == FieldProto.TAX_LOT_OPEN_DATE\
        #         or field_to_unpack.field == FieldProto.TAX_LOT_CLOSE_DATE:
        #     return LocalDateProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.IDENTIFIER:
        #     return IdentifierProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.TRANSACTION_TYPE:
        #     name:str = FieldProto.DESCRIPTOR.values_by_number[field_to_unpack.field].name
        #     return ProtoEnum(name, field_to_unpack.enum_value)
        # if field_to_unpack.field == FieldProto.PORTFOLIO_NAME or field_to_unpack.field == FieldProto.SECURITY_DESCRIPTION \
        #     or field_to_unpack.field == FieldProto.PRODUCT_TYPE:
        #     return wrappers.StringValue.FromString(field_to_unpack.field_value_packed.value).value
        # if field_to_unpack.field == FieldProto.PORTFOLIO:
        #     return PortfolioProto.FromString(field_to_unpack.field_value_packed.value)
        # if field_to_unpack.field == FieldProto.SECURITY:
        #     return SecurityProto.FromString(field_to_unpack.field_value_packed.value)

        # raise ValueError(f"Field not found. Could not unpack {field_to_unpack.field}")

    @staticmethod
    def unpack_field(field_to_unpack:FieldMapEntry):
        if field_to_unpack.field == FieldProto.PORTFOLIO_ID:
            return UUIDProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.TRADE_DATE or field_to_unpack.field == FieldProto.SETTLEMENT_DATE \
            or field_to_unpack.field == FieldProto.TAX_LOT_OPEN_DATE\
                or field_to_unpack.field == FieldProto.TAX_LOT_CLOSE_DATE:
            return LocalDateProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.IDENTIFIER:
            return IdentifierProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.TRANSACTION_TYPE:
            name:str = FieldProto.DESCRIPTOR.values_by_number[field_to_unpack.field].name
            return ProtoEnum(name, field_to_unpack.enum_value)
        if field_to_unpack.field == FieldProto.PORTFOLIO_NAME or field_to_unpack.field == FieldProto.SECURITY_DESCRIPTION \
            or field_to_unpack.field == FieldProto.PRODUCT_TYPE:
            return wrappers.StringValue.FromString(field_to_unpack.field_value_packed.value).value
        if field_to_unpack.field == FieldProto.PORTFOLIO:
            return PortfolioProto.FromString(field_to_unpack.field_value_packed.value)
        if field_to_unpack.field == FieldProto.SECURITY:
            return SecurityProto.FromString(field_to_unpack.field_value_packed.value)

        raise ValueError(f"Field not found. Could not unpack {field_to_unpack.field}")

    @staticmethod
    def unpack_measure(measure_to_unpack:MeasureProto):
        if measure_to_unpack.measure == MeasureProto.DIRECTED_QUANTITY:
            return Decimal(measure_to_unpack.measure_decimal_value.arbitrary_precision_value)
        
        raise ValueError(f"Field not found. Could not unpack {measure_to_unpack.field}")

